function code_151609_reconsAttIteratedScat(optsIn,filename,nIter,nScat)

opts = optsIn;

n = 32;
m = n/8; % number of direction between 0 and pi/4
thetas = pi/4*(1/2/m:1/m:8-1/2/m);


%--- characteristic function of the domain
domain = myMap('disk',n,0.9,0,0);

xs = -1+1/n:2/n:1-1/n;

%--- intrisic attenuation
% att = zeros(n);
att = myMap('mickey',n,0.02,12,0.2)+0.3*domain;
% att = myMap('nicebumps',n,0.1,5,0.5);%+0.3*domain;
% att = domain

%--- scattering
coeffScat = 0.2;
% scat = coeffScat*myMap('gaussian',n,1.5/n,0.5,0.5);
% scat = coeffScat*myMap('mickey',n,0.01,16,0.2);
% scat = coeffScat*myMap('phantom',n,0.06,3,0.5);
scat = coeffScat*(myMap('gaussian',n,0.3,0,0) + ...
    myMap('gaussian',n,0.125,0.5,0.5) + ...
    myMap('gaussian',n,0.125,-0.5,-0.5));

% cropping the coefficients to the domain
scat = scat.*domain;

%--- indices thIn, thOut
% indsIn = [1,1,1,1,1,1,1,1];%(1:7)*m;
% indsOut = indsIn + (1:8)*2*m/8;
indsIn = 1;
indsOut = indsIn + 2*m;

%--- input options
opts.input.type = 'dirac';
opts.input.sig = 4.5/n;

% nIter = 3; % number of iterations for the reconstruction scheme
% nScat = 4; % number of scattering terms for the forward solver

%--- diffusion types
%     opts.diff.type = 'linear'; opts.diff.d = tan(pi/4/m)^2;
%     opts.diff.type = 'FPB'; opts.diff.d = 0.0001;
%     opts.diff.type = 'none'; opts.diff.d = 0;

%--- domain for diffusion
opts.diff.domain = domain;
%--- recontruction formula for attenation
opts.diff.rc = 'peakCorr'; % 'int', 'peak', 'intCorr', 'peakCorr'

% corrector for scattering reconstruction based 
% on unitary scattering coefficient
opts.corr = 0;

%--- options for the inverse Radon Transform    
opts.IRT.filter = 'ram-lak'; % filter for the inverse Radon Transform
opts.IRT.band = 0.9; % frequency truncation for the IRT

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% MAIN

% figure; 
% subplot(1,3,1); dispmat(scat,'scattering');
% subplot(1,3,2); dispmat(att,'intrisic attenuation');
% subplot(1,3,3); dispmat(att+2*pi*scat,'total attenuation');
% 


% total attenuation
attTot = att + 2*pi*scat;

indIn = 1;
indOut  = 1+2*m;

%--- parameters for the inverse Radon Transform
filter = opts.IRT.filter;
band = opts.IRT.band;
%--- angles for the inverse Radon Transform
angles = 180/pi*(thetas-pi/2);
%--- number of radii for RT, make it even
nrRT = ceil(n*sqrt(2)) + mod(ceil(n*sqrt(2)),2);
%--- offset for the RT data
os = (nrRT-n)/2;

% diffusion
diff = makeDiff(n,opts.diff);

%--- allocate 
scatRc = zeros(n,n,nIter+1);
errScat = zeros(1,nIter+1);
attRc = zeros(n,n,nIter+1);
errAtt = zeros(1,nIter+1);

measCorr = zeros(n);

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ATTENUATION RECONSTRUCTION
disp('create the data')

[MAtt,MScat] = measForRcAttScat(n,m,thetas,attTot,scat,nScat,diff,...
    indIn,indOut,opts.input);

save(filename);

%%--- create ballistic data for total attenuation correction if necessary
calibration = ~strcmp(opts.diff.type,'none') && ...
    (strcmp(opts.diff.rc,'peakCorr') || strcmp(opts.diff.rc,'intCorr'));

if calibration
    disp('ballistic measurements without attenuation')
    corr = measForRcAtt(n,m,thetas,zeros(n),zeros(n),0,diff,0,opts.input);
    corr = measToRT(n,m,corr,opts.diff,opts.input);
 
    RT = [zeros(os,8*m); ...
        measToRT(n,m,MAtt,opts.diff,opts.input)-corr;zeros(os,8*m)];    
else
    RT = [zeros(os,8*m); ...
        measToRT(n,m,MAtt,opts.diff,opts.input);zeros(os,8*m)];    
end

attTotRc = (n/2)*myIRadon(RT,filter,angles,band,n);

% figure; 
% subplot(1,2,1); dispmat(attTotRc,'reconstructed')
% subplot(1,2,2); dispmat(attTot,'exact')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% NO ATTENUATION RECONSTRUCTION
% attTotRc = attTot;
% 
% MScat = measForRcScat(n,m,thetas,attTotRcRot,scat,nScat,diff,...
%     indIn,indOut,0,opts.input);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% create rotations of that map
attTotRcRot = rotateImages(attTotRc,thetas(1:2*m));

%%--- create corrector for scattering if necessary
if opts.corr
    measCorr = measForRcScat(n,m,thetas,attTotRcRot,ones(n),1,diff,...
        indIn,indOut,0,opts.input);
end

%%--- first guess for scattering
disp('first guess for scattering')

scatRc(:,:,1) = rcScat(n,m,thetas,attTotRc,MScat,indIn,indOut,...
    opts.corr,measCorr);

%%--- iterations scattering
for it = 1:nIter,
    disp(['Iteration ',num2str(it),' / ',num2str(nIter)])
    
    % create measurements with forward transport
    MScat = measForRcScat(n,m,thetas,attTotRcRot,scatRc(:,:,it), ...
        nScat,diff,indIn,indOut,1,opts.input);
    
    % reconstruction
    scatRc(:,:,it+1) = (scatRc(:,:,1) - rcScat(n,m,thetas,...
        attTotRc,MScat,indIn,indOut,opts.corr,measCorr) +...
        scatRc(:,:,it))/2;
end    
    
%%--- reconstructions intrinsic attenuation and errors
for it = 1:nIter+1,
    attRc(:,:,it) = attTotRc - 2*pi*scatRc(:,:,it);
    errScat(it) = norm(scatRc(:,:,it)-scat,'fro');
    errAtt(it) = norm(attRc(:,:,it)-att,'fro');
end

errAtt = errAtt/norm(att,'fro');
errScat = errScat/norm(scat,'fro');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% DISPLAY

save everythingDiff128;

% figure;
% for it=1:nIter,
%     subplot(2,nIter+1,it); dispmat(scatRc(:,:,it));
%     subplot(2,nIter+1,nIter+1+it);
%     dispmat(abs(scatRc(:,:,it+1)-scatRc(:,:,it)));
% end
% subplot(2,nIter+1,nIter+1); dispmat(scatRc(:,:,nIter+1));
% subplot(2,nIter+1,2*(nIter+1)); plot(errScat,'LineWidth',1.5);
% 
% figure;
% for it=1:nIter,
%     subplot(2,nIter+1,it); dispmat(attRc(:,:,it));
%     subplot(2,nIter+1,nIter+1+it);
%     dispmat(abs(attRc(:,:,it+1)-attRc(:,:,it)));
% end
% subplot(2,nIter+1,nIter+1); dispmat(attRc(:,:,nIter+1));
% subplot(2,nIter+1,2*(nIter+1)); plot(errAtt,'LineWidth',1.5);
