clc; clear; close all

n = 64;
m = n/8; % number of direction between 0 and pi/4
thetas = pi/4*(1/2/m:1/m:8-1/2/m);

xs = -1+1/n:2/n:1-1/n;

%--- characteristic function of the domain
domain = myMap('disk',n,1,0,0);

%--- intrisic attenuation
% att = zeros(n);
att = myMap('mickey',n,0.02,12,0.2);%+0.3*domain;
% att = myMap('nicebumps',n,0.1,5,0.5);%+0.3*domain;
% att = domain;

%--- scattering
coeffScat = 0.2;
% scat = coeffScat*myMap('gaussian',n,1.5/n,0.5,0.5);
% scat = coeffScat*myMap('mickey',n,0.01,16,0.2);
scat = coeffScat*myMap('nicebumps',n,0.1,3,0.4);
% scat = coeffScat*myMap('disk',n,0.03,0.5,0)...
%     + coeffScat*myMap('disk',n,0.03,-0.5,0);
% scat = coeffScat*(myMap('disk',n,0.5,0,0)...
%     -myMap('disk',n,0.4,0,0));
% scat = coeffScat*myMap('disk',n,0.02,0,0);
% scat = coeffScat*myMap('phantom',n,0.06,3,0.5);
% scat = coeffScat*(myMap('gaussian',n,0.3,0,0) + ...
%     myMap('gaussian',n,0.125,0.5,0.5) + ...
%     myMap('gaussian',n,0.125,-0.5,-0.5));
% scat = zeros(n);

% cropping the coefficients to the domain
scat = scat.*domain;

nIter = 0; % number of iterations for the reconstruction scheme
nScat = 3; % number of scattering terms for the forward solver

%--- indices thIn, thOut
% indsIn = [1,1,1,1];
% indsOut = indsIn + [m/4,ceil(2*m/3),ceil(4*m/3),2*m];
indsIn = 1;
indsOut = indsIn + m;

%--- input options
opts.input.type = 'dirac';
opts.input.sig = 4.5/n;

%--- diffusion types
%     opts.diff.type = 'linear'; opts.diff.d = tan(pi/4/m)^2;
    opts.diff.type = 'FPB'; opts.diff.d = 0.00;
  %   opts.diff.type = 'none'; opts.diff.sig = 0;

%--- domain for diffusion
opts.diff.domain = domain;
%--- recontruction formula for attenation
opts.diff.rc = 'intCorr'; % 'int', 'peak', 'intCorr', 'peakCorr'
    
% corrector for scattering reconstruction based 
% on unitary scattering coefficient
opts.corr = 0;

%--- options for the inverse Radon Transform    
opts.IRT.filter = 'ram-lak'; % filter for the inverse Radon Transform
opts.IRT.band = 0.9; % frequency truncation for the IRT

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% MAIN
% 
% figure; 
% subplot(1,3,1); dispmat(scat,'scattering');
% subplot(1,3,2); dispmat(att,'intrisic attenuation');
% subplot(1,3,3); dispmat(att+2*pi*scat,'total attenuation');
% 
% %% ============ one series of measurements
    attRot = rotateImages(att+2*pi*scat,thetas(1:2*m));

    diff = makeDiff(n,opts.diff);
    
    % localisation of the input ray
    indR = ceil(2*n/3); indTheta = m/2;
    
    input = makeInput(n,m,indR,indTheta,opts.input);
    
    tic
    meas = measurements(n,m,thetas,attRot,scat,diff,input,nScat,0);
    toc
    % --- display
    rs = -1+1/n:2/n:1-1/n;
    figure;
    subplot(2,3,1); dispmat(att,'attenuation')
    subplot(2,3,2); dispmat(scat,'scattering')
    subplot(2,3,3); plot(rs,meas(:,indTheta,1)); 
    title('ballistic part at \theta=\pi/8'); xlabel('r')
    subplot(2,3,4); dispmat(meas(:,:,2),'single scattering'); 
    xlabel('\theta'); ylabel('r')
    subplot(2,3,5); dispmat(meas(:,:,3),'double scattering'); 
    xlabel('\theta'); ylabel('r')
    subplot(2,3,6); dispmat(meas(:,:,4),'triple scattering'); 
    xlabel('\theta'); ylabel('r')

