function input = makeInput(n,m,indR,indTh,optsInput)
% this function creates the adequate inputs for the transport code based
% on image rotations and everything supported in the unit disk

input = zeros(n,8*m);
sig = optsInput.sig;

switch optsInput.type
    case 'dirac'
        input(indR,indTh) = (n/2)*(8*m/2/pi);
    case 'gaussian'
        xs = (-1+1/n:2/n:1-1/n).';
        input(:,indTh) = 1/sig/sqrt(2*pi)*exp(-(xs-xs(indR)).^2/2/sig^2)...
            *(8*m/2/pi);
    case 'isotropic' % does not depend on indR
        xs = (-1+1/n:2/n:1-1/n).';
        for j=-2*m:2*m,
            pos = -sin(j*pi/4/m);
            input(:,mod(j+indTh-1,8*m)+1) = ...
                1/sig/sqrt(2*pi)*exp(-(xs-pos).^2/2/sig^2);
        end
    case 'isotropic_weighted' % does not depend on indR
        xs = (-1+1/n:2/n:1-1/n).';
        for j=-2*m:2*m,
            pos = -sin(j*pi/4/m);
            input(:,mod(j+indTh-1,8*m)+1) = ...
                cos(j*pi/4/m)/sig/sqrt(2*pi)*exp(-(xs-pos).^2/2/sig^2);
        end
    otherwise
        error('The value for the type variable is not valid.')
end

end

