function meas = measurements(n,m,thetas,attRots,scat,diff,input,nScat,...
    indThOut)

% the diffusion is assumed to be constant throughout the unit disk
% in this case, the input is assumed to be a directional source
% - characterized by its position and angle of input
% - we assume no source term and isotropic scattering
%
% in this case, the rotated images for the attenuation are given for theta
% in [0,pi/2] and we deduce the others by rotations
%
% moreover, we expect to divide the number of rotations by 4 if we organize
% the loops correctly (rotations of the source term)
%
% to see old ideas, look for measurements2

%%--- initialization
meas = zeros(n,8*m,1+nScat);
inputnorms = sum(abs(input),1);
if nScat>0
    sumSol = zeros(n);
end

% sol = zeros(n);

%%--- computation of the ballistic part
% disp('ballistic part')

for th = 1:8*m,
    if inputnorms(th)
        %--- rotated attenuation map
        attRot = rot90(attRots(:,:,mod(th-1,2*m)+1),-floor((th-1)/2/m));

        %--- 1D propagation
        sol = solveODE(n,input(:,th),attRot,zeros(n),diff);
        meas(:,th,1) = sol(:,n);

        %--- contribution of this solution for the next source terms
        if nScat>0
            sumSol = sumSol + rotateImage(sol,-thetas(th));
        end
    end
end

% figure; dispmat(sumSol.*myMap('disk',n,1,0,0)); pause; close

%%--- computation of the next scattering events up to nScat-1

for s = 1:nScat-1,
%     disp(['scattering event # ',num2str(s),' / ',num2str(nScat)])
   
    %--- source term
    src = scat.*sumSol*2*pi/8/m;
    sumSol = zeros(n);    
        
    %--- loop over a quarter of all directions, 4 directions per iteration
    for th = 1:2*m,
%         if mod(th,10)==0 
%             disp(['th = ', num2str(th)])
%         end
        
        %--- compute the rotated images of attenuation and source for th
        attRot = attRots(:,:,th); 
        srcRot = rotateImage(src,thetas(th));
        
        %--- now compute the solution for direction th
        sol = solveODE(n,0,attRot,srcRot,0);
        
%         coco = rotateImage(sol,-thetas(th));
%         figure; dispmat(coco.*myMap('disk',n,1,0,0)); pause; close
            
        
        %--- update the measurements and the flux for the next iteration
        meas(:,th,s+1) = sol(:,n);
        sumSol = sumSol + rotateImage(sol,-thetas(th));
        
%         figure; dispmat(sumSol.*myMap('disk',n,1,0,0)); pause; close
        
        %------------
        for l=1:3,
            %--- compute the rotated images of attenuation and source 
            %--- for th + 2*m*l
            attRot = rot90(attRot,-1);
            srcRot = rot90(srcRot,-1);

            %--- now compute the solution for direction th + 2*m
            sol = solveODE(n,0,attRot,srcRot,0);
            
%             coco = rotateImage(sol,-thetas(th+2*m*l));
%             figure; dispmat(coco.*myMap('disk',n,1,0,0)); pause; close
                        
            %--- update the measurements and the flux for the next iteration
            meas(:,th+2*m*l,s+1) = sol(:,n);
            sumSol = sumSol + rotateImage(sol,-thetas(th+2*m*l));

        end
        
    end
    
%     figure; dispmat(sumSol.*myMap('disk',n,1,0,0)); pause; close
    
end %--- end loop over nScat

%%--- for the last iteration, compute only the directions we want
if nScat>=1
    
%     disp(['scattering event # ',num2str(nScat),' / ',num2str(nScat)])

    if all(~indThOut), %--- compute all directions in this case

        %--- source term
        src = scat.*sumSol*2*pi/8/m;
       
        %--- loop over a quarter of all directions, 4 directions per iteration
        for th = 1:2*m,
%             if mod(th,10)==0 
%                 disp(['th = ', num2str(th)])
%             end

            %--- compute the rotated images of attenuation and source for th
            attRot = attRots(:,:,th);
            srcRot = rotateImage(src,thetas(th));

            %--- now compute the solution for direction th
            sol = solveODE(n,0,attRot,srcRot,0);

            %--- update the measurements
            meas(:,th,nScat+1) = sol(:,n);

            %------------
            for l=1:3,
                %--- compute the rotated images of attenuation and source 
                %--- for th + 2*m*l
                attRot = rot90(attRot,-1);
                srcRot = rot90(srcRot,-1);

                %--- now compute the solution for direction th + 2*m
                sol = solveODE(n,0,attRot,srcRot,0);
                
                %--- update the measurements
                meas(:,th+2*m*l,nScat+1) = sol(:,n);
            end
        end
    else %--- compute only for the direction stored in indThOut
        for th = indThOut,
            
            %--- source term
            src = scat.*sumSol*2*pi/8/m;    
%             dispmat(src); pause
            
            %--- rotated attenuation map and source
            attRot = rot90(attRots(:,:,mod(th-1,2*m)+1),-floor((th-1)/2/m));
            srcRot = rotateImage(src,thetas(th));

            %--- 1D propagation
            sol = solveODE(n,0,attRot,srcRot,0);

            %--- update the measurements
            meas(:,th,nScat+1) = sol(:,n);
        end
    end
end
    
% displayMeas(meas,nScat)
% pause

end
%-------------------------------------------------------------------------

%=========================================================================
function displayMeas(meas,nScat)

figure;
for j=1:nScat+1,
    subplot(1,nScat+1,j); dispmat(meas(:,:,j));
end

end
%--------------------------------------------------------------------------