% function code_1118_integral_broken_lines()
% % this script test the partial integral function, and uses a benchmark case, 
% % a 2D Gaussian 
% %-------------------------------------------------------------------------- 
% % other functions :
% % - broken_line_integral
% % - partial_line_int
% % - partial_line_int_elem
% % - plot_unit_circle
% %--------------------------------------------------------------------------
% % external functions : predef_map
% 
% clc; clear; close all
% 
% n = 128;
% 
% %%--- benchmark to test the accurary of the method
% 
% % sig = 0.1;
% % att = predef_map('gaussian',n,sig,0.,0.,1);
% % 
% % r = 0.1;
% % theta = 3*pi/2+0.2;
% % 
% % pts = -.6:.02:.6;
% % xs = r*cos(theta) + pts*cos(theta+pi/2);
% % ys = r*sin(theta) + pts*sin(theta+pi/2);
% % 
% % ints = zeros(size(pts));
% % intex = zeros(size(pts));
% % for j=1:size(pts,2),
% %     ints(j) = partialLineInt(att,n,xs(j),ys(j),theta+pi/2);
% %     intex(j) = exp(-r^2/2/sig^2)*sig*sqrt(pi/2)*(erf(pts(j)/sig/sqrt(2))-erf(-.5/sig/sqrt(2)));
% % end
% % 
% % %%--- display
% % figure; 
% % subplot(1,3,1); plot(xs,ys,'x'); hold on; plotUnitCircle(); axis equal
% % subplot(1,3,2); plot(pts,ints,'b',pts,intex,'r'); legend('computed','exact')
% % subplot(1,3,3); surf(att); shading flat
% 
% %%--- test all directions
% 
% m = n/8;
% thetas = pi/4*(1/2/m:1/m:8-1/2/m)
% 
% att2 = predef_map('mickey',n,0.,0.,0.,1);
% 
% figure;
% for j=1:8*m,
%     j
%     pLI = partialLineInts(att2,n,thetas(j));
%     dispmat(pLI); pause
% end
% 
% end

% function partialLineInts
%-------------------------------------------------------------------------
function res = partialLineInts(im,n,theta)
    % this function computes the partial line integrals on an image along
    % a direction theta, on the regular mesh of the square [-1,1]^2
    %------------
    % uses partialLineIntElem
    res = zeros(n);
    
    xs = -1+1/n:2/n:1-1/n;
    ys = xs;
    theta = mod(theta,2*pi);
    
    if theta==0
        res = fliplr(2/n*cumsum(fliplr(im),2));
    elseif theta<pi/4
        for i=1:n,
            for j=1:n,
                res(i,j) = partialLineIntElem(im,n,xs(j),ys(i),theta);
            end
        end
    elseif theta<pi/2
        imRot = flipud(rot90(im,1));
        for i=1:n,
            for j=1:n,
                res(i,j) = partialLineIntElem(imRot,n,ys(i),xs(j),pi/2-theta);
            end
        end
    elseif theta==pi/2
        res = 2/n*cumsum(im,1);
    elseif theta<3*pi/4
        imRot = rot90(im,1);
        for i=1:n,
            for j=1:n,
                res(i,j) = partialLineIntElem(imRot,n,ys(i),-xs(j),theta-pi/2);
            end
        end
    elseif theta<pi
        imRot = flipud(rot90(im,2));
        for i=1:n,
            for j=1:n,
                res(i,j) = partialLineIntElem(imRot,n,-xs(j),ys(i),pi-theta);
            end
        end
    elseif theta==pi
        res = 2/n*cumsum(im,2);
    elseif theta<5*pi/4
        imRot = rot90(im,2);
        for i=1:n,
            for j=1:n,
                res(i,j) = partialLineIntElem(imRot,n,-xs(j),-ys(i),theta-pi);
            end
        end
    elseif theta<3*pi/2
        imRot = flipud(rot90(im,3));
        for i=1:n,
            for j=1:n,
                res(i,j) = partialLineIntElem(imRot,n,-ys(i),-xs(j),3*pi/2-theta);
            end
        end
    elseif theta==3*pi/2
        res = flipud(2/n*cumsum(flipud(im),1));
    elseif theta<7*pi/4
        imRot = rot90(im,3);
        for i=1:n,
            for j=1:n,
                res(i,j) = partialLineIntElem(imRot,n,-ys(i),xs(j),theta-3*pi/2);
            end
        end
    else
        imRot = flipud(im);
        for i=1:n,
            for j=1:n,
                res(i,j) = partialLineIntElem(imRot,n,xs(j),-ys(i),2*pi-theta);
            end
        end
    end  
   
end

% function brokenLineInt
%-------------------------------------------------------------------------
function res = brokenLineInt(im,n,x,y,thIn,thOut)
    % this function computes the integral of a field alog a broken line,
    % along direction thIn up to (x,y), then from (x,y) in direction
    % thOut
    
    res = partialLineInt(im,n,x,y,thIn) ...
        + partialLineInt(im,n,x,y,thOut + pi);
    
end

% function partialLineInt
%-------------------------------------------------------------------------
function res = partialLineInt(im,n,x,y,theta)
    % this function computes the partial line integral on an image along
    % a direction theta, up to the point (x,y)
    
    theta = mod(theta,2*pi);
    if theta<pi/4
        res = partialLineIntElem(im,n,x,y,theta);
    elseif theta<pi/2
        res = partialLineIntElem(flipud(rot90(im,-1)),n,y,x,pi/2-theta);
    elseif theta<3*pi/4
        res = partialLineIntElem(rot90(im,-1),n,y,-x,theta-pi/2);
    elseif theta<pi
        res = partialLineIntElem(flipud(rot90(im,-2)),n,-x,y,pi-theta);
    elseif theta<5*pi/4
        res = partialLineIntElem(rot90(im,-2),n,-x,-y,theta-pi);
    elseif theta<3*pi/2
        res = partialLineIntElem(flipud(rot90(im,-3)),n,-y,-x,3*pi/2-theta);
    elseif theta<7*pi/4
        res = partialLineIntElem(rot90(im,-3),n,-y,x,theta-3*pi/2);
    else
        res = partialLineIntElem(flipud(im),n,x,-y,2*pi-theta);
    end
    
    
end

% function partialLineIntElem
%-------------------------------------------------------------------------
function res = partialLineIntElem(im,n,x,y,theta)
    % in this elementary case, theta is assumed to be in (0,pi/4)
    
    if theta<0 || theta>pi/4,
        error('the value for theta is not valid.')
    end
    
    xs = -1+1/n:2/n:1-1/n;

    jmin = max(1,floor(1+n/2*(x+1-1/n-1/tan(theta)*(y+1-1/n)))+1);
    
    res = 0;
    for j=jmin:floor(n/2*(x+1-1/n))+1,
        yj = y + (xs(j)-x)*tan(theta);
        yjp = n/2*(yj+1-1/n)+1;
        if floor(yjp)<n
            res = res + im(floor(yjp),j)*(1-mod(yjp,1)) ...
                + im(floor(yjp)+1,j)*mod(yjp,1);
        end
    end
    
    % weight of each point (length of elementary interval of integration)
    res = 2/n/cos(theta)*res;   
    
end

% function plotUnitCircle
%-------------------------------------------------------------------------
function plotUnitCircle()

    n = 100;
    ts = 2*pi*(0:1/n:1);
    xs = cos(ts);
    ys = sin(ts);

    plot(xs,ys,'r'); hold on; plot(0.,0.,'ro')

end

