function scatRc = rcScat(n,m,thetas,att,meas,indIn,indOut,corr,measCorr)
% reconstructs the scattering coefficient given the appropriate 
% measurements.
% the corr boolean tells whether or not the measurements have been divided
% by the same measurements with normalized scattering

indIn = mod(indIn - 1,8*m)+1;
indOut = mod(indOut - 1,8*m)+1;

scatRcRot = zeros(n);

%--- rotate the attenuation into the frame of thetas(indIn)
attRot = rotateImage(att,thetas(indIn));

%--- number of increments between thIn and thOut
d_ind = mod(indOut - indIn - 1,8*m)+1;

alpha = d_ind*pi/4/m; % deviation
   
xs = -1+1/n:2/n:1-1/n;
ys = xs';   

if corr % using corrected measurements
    for indR = 1:n,
        for j=1:n,
            r = -xs(n+1-j)*sin(alpha) + ys(indR)*cos(alpha);
            temp = specInterp(n,r-xs);
            scatRcRot(indR,j) = (temp*meas(:,indR))/...
                (temp*measCorr(:,indR));
        end
    end
else
    %--- partial integrals along thIn
    attSum1 = exp(2/n*cumsum(attRot,2));
    %--- partial integrals along thOut
    attSum2 = exp(partialLineInts(attRot,n,pi-alpha));
    %--- resulting broken line attenuation    
    attSum = attSum1.*attSum2;

    %--- reconstruction formula
    for indR = 1:n,
        for j=1:n,
            r = -xs(n+1-j)*sin(alpha) + ys(indR)*cos(alpha);
            temp = specInterp(n,r-xs);
            scatRcRot(indR,j) = (temp*meas(:,indR))*attSum(indR,j);
        end
    end
    
    % scaling coefficient
    scatRcRot = scatRcRot*abs(sin(alpha));
end

%--- rotate back the reconstruction
if corr
    scatRcRot = scatRcRot.*myMap('disk',n,0.95,0,0);
end

scatRc = rotateImage(scatRcRot,-thetas(indIn));

end